@@ -2,6 +2,7 @@ from __future__ import annotations
 from datetime import datetime
 from .paths import paths
 from selenium import webdriver
+from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.by import By
 from selenium.webdriver.chrome.options import Options
 from selenium.webdriver.remote.webelement import WebElement
@@ -10,11 +11,11 @@ from .variables.variables import *
 
 
 class User:
-    def __init__(self, name: str, password: str, session: Session, role=Roles.STUDENT) -> None:
+    def __init__(self, name: str, password: str, session: Session) -> None:
         self.name = name
         self.password = password
         self.session = session
-        self.role = role
+        self.role = Roles.from_username(name)
 
     @property
     def schoolpass(self) -> int:
@@ -28,8 +29,8 @@ class User:
 
 
 class Session:
-    def __init__(self, user: str, password: str, chromedriver_path: str=r"C:\Chromedriver\chromedriver.exe", role: str=Roles.STUDENT, hidden: bool=False) -> None:
-        self.user = User(user, password, self, role)
+    def __init__(self, user: str, password: str, chromedriver_path: str=r"C:\Chromedriver\chromedriver.exe", hidden: bool=False) -> None:
+        self.user = User(user, password, self)
         try:
             chrome_options = Options()
             if (hidden):
@@ -113,7 +114,8 @@ class Valutazioni(Finestra):
                 last_subject = tr.find_element(By.TAG_NAME, "td").text
             else:
                 tds: list[WebElement] = tr.find_elements(By.TAG_NAME, "td")
-                mark: str = tds[2].find_elements(By.TAG_NAME, "div")[0].find_element(By.TAG_NAME, "p").text
+                td: WebElement = tds[2].find_elements(By.TAG_NAME, "div")[0]
+                mark: str = td.find_element(By.TAG_NAME, "p").text
                 if (not (any(sym in mark for sym in {'+', '-'}) or mark == 'g')):
                     mark = float(mark.replace('½', '.5'))
                 res = [last_subject, mark]
@@ -142,7 +144,8 @@ class Valutazioni(Finestra):
                 last_subject = tr.find_element(By.TAG_NAME, "td").text
                 result[last_subject] = []
             else:
-                mark: str = tds[2].find_elements(By.TAG_NAME, "div")[0].find_element(By.TAG_NAME, "p").text
+                td: WebElement = tds[2].find_elements(By.TAG_NAME, "div")[0]
+                mark: str = td.find_element(By.TAG_NAME, "p").text
                 if (not (any(sym in mark for sym in {'+', '-'}) or mark == 'g')):
                     mark = float(mark.replace('½', '.5'))
                 res = [last_subject, mark]
@@ -152,7 +155,8 @@ class Valutazioni(Finestra):
                 if (type_):
                     res.append(tds[3].find_element(By.TAG_NAME, "p").get_attribute("title"))
                 if (notes):
-                    res.append(tds[5].find_element(By.TAG_NAME, "div").find_element(By.TAG_NAME, "span").text)
+                    td_: WebElement = tds[5].find_element(By.TAG_NAME, "div")
+                    res.append(td_.find_element(By.TAG_NAME, "span").text)
                 result[last_subject].append(tuple(res))
         return result
 
@@ -177,7 +181,7 @@ class Note(Finestra):
         counter += len(self.driver.find_elements(By.XPATH, paths.nota_trs))
         return counter
 
-    def get_notes(self, sort: int=NoteSortBy.AUTHOR, teacher: bool=True, date: bool=True, type_: bool=True) -> list[tuple[str, str, str, str] | tuple[str, str, str] | tuple[str, str] | tuple[str]]:
+    def get_notes(self, sort: int=NoteSortBy.AUTHOR, read: bool=False, teacher: bool=True, date: bool=True, type_: bool=True) -> list[tuple[str, str, str, str] | tuple[str, str, str] | tuple[str, str] | tuple[str]]:
         if (self.driver.current_url != paths.note_url):
             self.driver.get(paths.note_url)
         # Select the specified sorting
@@ -188,6 +192,13 @@ class Note(Finestra):
         for tr in self.driver.find_elements(By.XPATH, paths.nota_trs):
             tds: list[WebElement] = tr.find_elements(By.TAG_NAME, "td")
             res = [tds[2].text]
+            if (read):
+                if (tds[2].text.lower().strip() == "leggi"): # If the note is unread...
+                    actions: ActionChains = ActionChains(self.driver)
+                    actions.move_to_element(tds[2])
+                    actions.click() # Click over the element to read the note
+                    actions.perform()
+                    res[0] = tds[2].text # Update the title
             if (teacher):
                 res.append(tds[0].text)
             if (date):
@@ -197,6 +208,9 @@ class Note(Finestra):
             result.append(tuple(res))
         return result
 
+    def read_notes(self) -> None:
+        ...
+
 
 class Registro(Finestra):
     def __init__(self, session: Session) -> None:
